@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -29,7 +29,8 @@
 # include <wchar.h>
 #endif
 
-#if defined(HAVE_ASPRINTF_L) || defined(HAVE_USELOCALE)
+#if defined(HAVE_NEWLOCALE) && \
+    (defined(HAVE_ASPRINTF_L) || defined(HAVE_USELOCALE))
 # include <locale.h>
 #endif
 #ifdef HAVE_XLOCALE_H
@@ -41,8 +42,9 @@
 #endif
 
 #import "OFASPrintF.h"
-#import "OFString.h"
 #import "OFLocale.h"
+#import "OFString.h"
+#import "OFString+Private.h"
 
 #import "OFInitializationFailedException.h"
 
@@ -88,7 +90,8 @@ struct Context {
 	bool useLocale;
 };
 
-#if defined(HAVE_ASPRINTF_L) || defined(HAVE_USELOCALE)
+#if defined(HAVE_NEWLOCALE) && \
+    (defined(HAVE_ASPRINTF_L) || defined(HAVE_USELOCALE))
 static locale_t cLocale;
 
 OF_CONSTRUCTOR()
@@ -344,7 +347,7 @@ formatLengthModifierState(struct Context *ctx)
 
 		break;
 #ifdef OF_WINDOWS
-	case 'I': /* win32 strangeness (I64 instead of ll or j) */
+	case 'I': /* Win32 strangeness (I64 instead of ll or j) */
 		if (ctx->formatLen > ctx->i + 2 &&
 		    ctx->format[ctx->i + 1] == '6' &&
 		    ctx->format[ctx->i + 2] == '4') {
@@ -382,7 +385,8 @@ formatConversionSpecifierState(struct Context *ctx)
 {
 	char *tmp = NULL;
 	int tmpLen = 0;
-#if !defined(HAVE_ASPRINTF_L) && !defined(HAVE_USELOCALE)
+#if !defined(HAVE_NEWLOCALE) || \
+    (!defined(HAVE_ASPRINTF_L) && !defined(HAVE_USELOCALE))
 	OFString *point;
 #endif
 
@@ -423,7 +427,7 @@ formatConversionSpecifierState(struct Context *ctx)
 
 		{
 			char buffer[5];
-			size_t len = OFUTF8StringEncode(
+			size_t len = _OFUTF8StringEncode(
 			    va_arg(ctx->arguments, OFUnichar), buffer);
 
 			if (len == 0)
@@ -454,15 +458,15 @@ formatConversionSpecifierState(struct Context *ctx)
 
 			j = 0;
 			for (size_t i = 0; i < len; i++) {
-				size_t clen = OFUTF8StringEncode(arg[i],
+				size_t cLen = _OFUTF8StringEncode(arg[i],
 				    buffer + j);
 
-				if (clen == 0) {
+				if (cLen == 0) {
 					free(buffer);
 					return false;
 				}
 
-				j += clen;
+				j += cLen;
 			}
 			buffer[j] = 0;
 
@@ -553,13 +557,13 @@ formatConversionSpecifierState(struct Context *ctx)
 		switch (ctx->lengthModifier) {
 		case lengthModifierNone:
 		case lengthModifierL:
-#if defined(HAVE_ASPRINTF_L)
+#if defined(HAVE_NEWLOCALE) && defined(HAVE_ASPRINTF_L)
 			if (!ctx->useLocale)
 				tmpLen = asprintf_l(&tmp, cLocale,
 				    ctx->subformat,
 				    va_arg(ctx->arguments, double));
 			else
-#elif defined(HAVE_USELOCALE)
+#elif defined(HAVE_NEWLOCALE) && defined(HAVE_USELOCALE)
 			if (!ctx->useLocale) {
 				locale_t previousLocale = uselocale(cLocale);
 				tmpLen = asprintf(&tmp, ctx->subformat,
@@ -571,13 +575,13 @@ formatConversionSpecifierState(struct Context *ctx)
 				    va_arg(ctx->arguments, double));
 			break;
 		case lengthModifierCapitalL:
-#if defined(HAVE_ASPRINTF_L)
+#if defined(HAVE_NEWLOCALE) && defined(HAVE_ASPRINTF_L)
 			if (!ctx->useLocale)
 				tmpLen = asprintf_l(&tmp, cLocale,
 				    ctx->subformat,
 				    va_arg(ctx->arguments, long double));
 			else
-#elif defined(HAVE_USELOCALE)
+#elif defined(HAVE_NEWLOCALE) && defined(HAVE_USELOCALE)
 			if (!ctx->useLocale) {
 				locale_t previousLocale = uselocale(cLocale);
 				tmpLen = asprintf(&tmp, ctx->subformat,
@@ -592,7 +596,8 @@ formatConversionSpecifierState(struct Context *ctx)
 			return false;
 		}
 
-#if !defined(HAVE_ASPRINTF_L) && !defined(HAVE_USELOCALE)
+#if !defined(HAVE_NEWLOCALE) || \
+    (!defined(HAVE_ASPRINTF_L) && !defined(HAVE_USELOCALE))
 		if (tmpLen == -1)
 			return false;
 
@@ -759,7 +764,7 @@ static bool (*states[])(struct Context *) = {
 };
 
 int
-OFVASPrintF(char **string, const char *format, va_list arguments)
+_OFVASPrintF(char **string, const char *format, va_list arguments)
 {
 	struct Context ctx;
 
