@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -340,8 +340,7 @@ convert(OFMutableString *self, char (*startFunction)(char),
 	va_list arguments;
 
 	va_start(arguments, format);
-	[self appendFormat: format
-		 arguments: arguments];
+	[self appendFormat: format arguments: arguments];
 	va_end(arguments);
 }
 
@@ -353,7 +352,7 @@ convert(OFMutableString *self, char (*startFunction)(char),
 	if (format == nil)
 		@throw [OFInvalidArgumentException exception];
 
-	if ((UTF8StringLength = OFVASPrintF(&UTF8String, format.UTF8String,
+	if ((UTF8StringLength = _OFVASPrintF(&UTF8String, format.UTF8String,
 	    arguments)) == -1)
 		@throw [OFInvalidFormatException exception];
 
@@ -367,26 +366,26 @@ convert(OFMutableString *self, char (*startFunction)(char),
 #ifdef OF_HAVE_UNICODE_TABLES
 - (void)uppercase
 {
-	[self of_convertWithWordStartTable: OFUnicodeUppercaseTable
-			   wordMiddleTable: OFUnicodeUppercaseTable
-			wordStartTableSize: OFUnicodeUppercaseTableSize
-		       wordMiddleTableSize: OFUnicodeUppercaseTableSize];
+	[self of_convertWithWordStartTable: _OFUnicodeUppercaseTable
+			   wordMiddleTable: _OFUnicodeUppercaseTable
+			wordStartTableSize: _OFUnicodeUppercaseTableSize
+		       wordMiddleTableSize: _OFUnicodeUppercaseTableSize];
 }
 
 - (void)lowercase
 {
-	[self of_convertWithWordStartTable: OFUnicodeLowercaseTable
-			   wordMiddleTable: OFUnicodeLowercaseTable
-			wordStartTableSize: OFUnicodeLowercaseTableSize
-		       wordMiddleTableSize: OFUnicodeLowercaseTableSize];
+	[self of_convertWithWordStartTable: _OFUnicodeLowercaseTable
+			   wordMiddleTable: _OFUnicodeLowercaseTable
+			wordStartTableSize: _OFUnicodeLowercaseTableSize
+		       wordMiddleTableSize: _OFUnicodeLowercaseTableSize];
 }
 
 - (void)capitalize
 {
-	[self of_convertWithWordStartTable: OFUnicodeTitlecaseTable
-			   wordMiddleTable: OFUnicodeLowercaseTable
-			wordStartTableSize: OFUnicodeTitlecaseTableSize
-		       wordMiddleTableSize: OFUnicodeLowercaseTableSize];
+	[self of_convertWithWordStartTable: _OFUnicodeTitlecaseTable
+			   wordMiddleTable: _OFUnicodeLowercaseTable
+			wordStartTableSize: _OFUnicodeTitlecaseTableSize
+		       wordMiddleTableSize: _OFUnicodeLowercaseTableSize];
 }
 #else
 - (void)uppercase
@@ -444,8 +443,7 @@ convert(OFMutableString *self, char (*startFunction)(char),
 	if (string == nil || replacement == nil)
 		@throw [OFInvalidArgumentException exception];
 
-	if (range.length > SIZE_MAX - range.location ||
-	    range.location + range.length > self.length)
+	if (OFEndOfRange(range) > self.length)
 		@throw [OFOutOfRangeException exception];
 
 	if (searchLength > range.length) {
@@ -529,6 +527,57 @@ convert(OFMutableString *self, char (*startFunction)(char),
 	[self deleteTrailingWhitespaces];
 }
 
+- (void)replaceControlCharacters
+{
+	void *pool;
+	const OFUnichar *characters;
+	size_t length = self.length;
+
+	if (length == 0)
+		return;
+
+	pool = objc_autoreleasePoolPush();
+	characters = self.characters;
+
+	for (size_t i = 0; i < length; i++) {
+		if (characters[i] <= 0x1F) {
+			OFString *replacement = [OFString
+			    stringWithFormat: @"%C", characters[i] + 0x2400];
+
+			[self replaceCharactersInRange: OFMakeRange(i, 1)
+					    withString: replacement];
+
+			objc_autoreleasePoolPop(pool);
+			pool = objc_autoreleasePoolPush();
+
+			characters = self.characters;
+		} else if (characters[i] == 0x7F) {
+			[self replaceCharactersInRange: OFMakeRange(i, 1)
+					    withString: @"␡"];
+
+			objc_autoreleasePoolPop(pool);
+			pool = objc_autoreleasePoolPush();
+
+			characters = self.characters;
+		} else if (characters[i] >= 0x80 && characters[i] <= 0x9F) {
+			OFString *replacement = [OFString
+			    stringWithFormat: @"␛%C", characters[i] - 0x40];
+
+			[self replaceCharactersInRange: OFMakeRange(i, 1)
+					    withString: replacement];
+
+			objc_autoreleasePoolPop(pool);
+			pool = objc_autoreleasePoolPush();
+
+			characters = self.characters;
+			length++;
+			i++;
+		}
+	}
+
+	objc_autoreleasePoolPop(pool);
+}
+
 - (id)copy
 {
 	return [[OFString alloc] initWithString: self];
