@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -153,7 +153,7 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  * use the result outside the scope of the current autorelease pool, you have to
  * copy it.
  */
-@property (readonly, nonatomic) const char *UTF8String;
+@property (readonly, nonatomic) const char *UTF8String OF_RETURNS_INNER_POINTER;
 
 /**
  * @brief The number of bytes the string needs in UTF-8 encoding.
@@ -179,29 +179,113 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  */
 @property (readonly, nonatomic) OFString *capitalizedString;
 
+/**
+ * @brief The decimal value of the string as a `char`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big or too small to fit into
+ *				a `char`
+ */
+@property (readonly, nonatomic) signed char charValue;
+
+/**
+ * @brief The decimal value of the string as a `short`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big or too small to fit into
+ *				a `short`
+ */
+@property (readonly, nonatomic) short shortValue;
+
+/**
+ * @brief The decimal value of the string as an `int`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big or too small to fit into
+ *				an `int`
+ */
+@property (readonly, nonatomic) int intValue;
+
+/**
+ * @brief The decimal value of the string as a `long`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big or too small to fit into
+ *				a `long`
+ */
+@property (readonly, nonatomic) long longValue;
+
 /**
  * @brief The decimal value of the string as a `long long`.
  *
  * Leading and trailing whitespaces are ignored.
  *
- * If the string contains any non-number characters, an
- * @ref OFInvalidFormatException is thrown.
- *
- * If the number is too big to fit into a `long long`, an
- * @ref OFOutOfRangeException is thrown.
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big or too small to fit into
+ *				a `long long`
  */
 @property (readonly, nonatomic) long long longLongValue;
 
+/**
+ * @brief The decimal value of the string as an `unsigned char`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big to fit into an
+ *				`unsigned char`
+ */
+@property (readonly, nonatomic) unsigned char unsignedCharValue;
+
+/**
+ * @brief The decimal value of the string as an `unsigned short`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big to fit into an
+ *				`unsigned short`
+ */
+@property (readonly, nonatomic) unsigned short unsignedShortValue;
+
+/**
+ * @brief The decimal value of the string as an `unsigned int`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big to fit into an
+ *				`unsigned int`
+ */
+@property (readonly, nonatomic) unsigned int unsignedIntValue;
+
+/**
+ * @brief The decimal value of the string as an `unsigned long`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big to fit into an
+ *				`unsigned long`
+ */
+@property (readonly, nonatomic) unsigned long unsignedLongValue;
+
 /**
  * @brief The decimal value of the string as an `unsigned long long`.
  *
  * Leading and trailing whitespaces are ignored.
  *
- * If the string contains any non-number characters, an
- * @ref OFInvalidFormatException is thrown.
- *
- * If the number is too big to fit into an `unsigned long long`, an
- * @ref OFOutOfRangeException is thrown.
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big to fit into an
+ *				`unsigned long long`
  */
 @property (readonly, nonatomic) unsigned long long unsignedLongLongValue;
 
@@ -230,7 +314,8 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  *
  * The returned string is *not* null-terminated.
  */
-@property (readonly, nonatomic) const OFUnichar *characters;
+@property (readonly, nonatomic) const OFUnichar *characters
+    OF_RETURNS_INNER_POINTER;
 
 /**
  * @brief The string in UTF-16 encoding with native byte order.
@@ -241,7 +326,8 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  *
  * The returned string is null-terminated.
  */
-@property (readonly, nonatomic) const OFChar16 *UTF16String;
+@property (readonly, nonatomic) const OFChar16 *UTF16String
+    OF_RETURNS_INNER_POINTER;
 
 /**
  * @brief The length of the string in UTF-16 characters.
@@ -257,7 +343,8 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  *
  * The returned string is null-terminated.
  */
-@property (readonly, nonatomic) const OFChar32 *UTF32String;
+@property (readonly, nonatomic) const OFChar32 *UTF32String
+    OF_RETURNS_INNER_POINTER;
 
 /**
  * @brief The string with leading whitespaces deleted.
@@ -274,6 +361,12 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  */
 @property (readonly, nonatomic) OFString *stringByDeletingEnclosingWhitespaces;
 
+/**
+ * @brief The string with all control characters replaced with a description of
+ *	  the replaced control character.
+ */
+@property (readonly, nonatomic) OFString *stringByReplacingControlCharacters;
+
 #if defined(OF_WINDOWS) || defined(DOXYGEN)
 /**
  * @brief The string with the Windows Environment Strings expanded.
@@ -907,7 +1000,8 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  * @throw OFInvalidEncodingException The string cannot be represented in the
  *				     specified encoding
  */
-- (const char *)cStringWithEncoding: (OFStringEncoding)encoding;
+- (const char *)cStringWithEncoding: (OFStringEncoding)encoding
+    OF_RETURNS_INNER_POINTER;
 
 /**
  * @brief Returns the OFString as a C string in the specified encoding,
@@ -921,7 +1015,8 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  * @param encoding The encoding for the C string
  * @return The OFString as a C string in the specified encoding
  */
-- (const char *)lossyCStringWithEncoding: (OFStringEncoding)encoding;
+- (const char *)lossyCStringWithEncoding: (OFStringEncoding)encoding
+    OF_RETURNS_INNER_POINTER;
 
 /**
  * @brief Returns the OFString as an insecure C string (meaning it can contain
@@ -934,7 +1029,8 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  * @param encoding The encoding for the C string
  * @return The OFString as a C string in the specified encoding
  */
-- (const char *)insecureCStringWithEncoding: (OFStringEncoding)encoding;
+- (const char *)insecureCStringWithEncoding: (OFStringEncoding)encoding
+    OF_RETURNS_INNER_POINTER;
 
 /**
  * @brief Returns the number of bytes the string needs in the specified
@@ -1004,7 +1100,7 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  * @brief Returns the range of the string in the specified range.
  *
  * @param string The string to search
- * @param options Options modifying search behaviour
+ * @param options Options modifying search behavior
  * @param range The range in which to search
  * @return The range of the first occurrence of the string or a range with
  *	   `OFNotFound` as start position if it was not found
@@ -1013,6 +1109,39 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
 		 options: (OFStringSearchOptions)options
 		   range: (OFRange)range;
 
+/**
+ * @brief Returns the range of the first character from the set.
+ *
+ * @param characterSet The set of characters to search for
+ * @return The range of the first occurrence of a character from the set or a
+ *	   range with `OFNotFound` as start position if it was not found
+ */
+- (OFRange)rangeOfCharacterFromSet: (OFCharacterSet *)characterSet;
+
+/**
+ * @brief Returns the range of the first character from the set.
+ *
+ * @param characterSet The set of characters to search for
+ * @param options Options modifying search behavior
+ * @return The range of the first occurrence of a character from the set or a
+ *	   range with `OFNotFound` as start position if it was not found
+ */
+- (OFRange)rangeOfCharacterFromSet: (OFCharacterSet *)characterSet
+			   options: (OFStringSearchOptions)options;
+
+/**
+ * @brief Returns the index of the first character from the set.
+ *
+ * @param characterSet The set of characters to search for
+ * @param options Options modifying search behavior
+ * @param range The range in which to search
+ * @return The range of the first occurrence of a character from the set or a
+ *	   range with `OFNotFound` as start position if it was not found
+ */
+- (OFRange)rangeOfCharacterFromSet: (OFCharacterSet *)characterSet
+			   options: (OFStringSearchOptions)options
+			     range: (OFRange)range;
+
 /**
  * @brief Returns the index of the first character from the set.
  *
@@ -1020,7 +1149,8 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  * @return The index of the first occurrence of a character from the set or
  *	   `OFNotFound` if it was not found
  */
-- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet;
+- (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
+    OF_DEPRECATED(ObjFW, 1, 3, "Use -[rangeOfCharacterFromSet:] instead");
 
 /**
  * @brief Returns the index of the first character from the set.
@@ -1031,7 +1161,9 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  *	   `OFNotFound` if it was not found
  */
 - (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
-			  options: (OFStringSearchOptions)options;
+			  options: (OFStringSearchOptions)options
+    OF_DEPRECATED(ObjFW, 1, 3,
+	"Use -[rangeOfCharacterFromSet:options:] instead");
 
 /**
  * @brief Returns the index of the first character from the set.
@@ -1044,7 +1176,9 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  */
 - (size_t)indexOfCharacterFromSet: (OFCharacterSet *)characterSet
 			  options: (OFStringSearchOptions)options
-			    range: (OFRange)range;
+			    range: (OFRange)range
+    OF_DEPRECATED(ObjFW, 1, 3,
+	"Use -[rangeOfCharacterFromSet:options:range:] instead");
 
 /**
  * @brief Returns whether the string contains the specified string.
@@ -1078,47 +1212,163 @@ typedef void (^OFStringLineEnumerationBlock)(OFString *line, bool *stop);
  */
 - (OFString *)substringWithRange: (OFRange)range;
 
+/**
+ * @brief The value of the string in the specified base as a `char`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @param base The base to use. If the base is 0, base 16 is assumed if the
+ *	       string starts with 0x (after stripping white spaces). If the
+ *	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
+ *	       assumed.
+ * @return The value of the string in the specified base
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big or too small to fit into
+ *				a `char`
+ */
+- (signed char)charValueWithBase: (unsigned char)base;
+
+/**
+ * @brief The value of the string in the specified base as a `short`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @param base The base to use. If the base is 0, base 16 is assumed if the
+ *	       string starts with 0x (after stripping white spaces). If the
+ *	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
+ *	       assumed.
+ * @return The value of the string in the specified base
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big or too small to fit into
+ *				a `short`
+ */
+- (short)shortValueWithBase: (unsigned char)base;
+
+/**
+ * @brief The value of the string in the specified base as an `int`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @param base The base to use. If the base is 0, base 16 is assumed if the
+ *	       string starts with 0x (after stripping white spaces). If the
+ *	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
+ *	       assumed.
+ * @return The value of the string in the specified base
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big or too small to fit into
+ *				an `int`
+ */
+- (int)intValueWithBase: (unsigned char)base;
+
+/**
+ * @brief The value of the string in the specified base as a `long`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @param base The base to use. If the base is 0, base 16 is assumed if the
+ *	       string starts with 0x (after stripping white spaces). If the
+ *	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
+ *	       assumed.
+ * @return The value of the string in the specified base
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big or too small to fit into
+ *				a `long`
+ */
+- (long)longValueWithBase: (unsigned char)base;
+
 /**
  * @brief The value of the string in the specified base as a `long long`.
  *
  * Leading and trailing whitespaces are ignored.
  *
- * If the string contains any non-number characters, an
- * @ref OFInvalidFormatException is thrown.
- *
- * If the number is too big to fit into a `long long`, an
- * @ref OFOutOfRangeException is thrown.
- *
  * @param base The base to use. If the base is 0, base 16 is assumed if the
- * 	       string starts with 0x (after stripping white spaces). If the
- * 	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
- * 	       assumed.
+ *	       string starts with 0x (after stripping white spaces). If the
+ *	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
+ *	       assumed.
  * @return The value of the string in the specified base
- * @throw OFInvalidFormatException The string cannot be parsed as a `long long`
- * @throw OFOutOfRangeException The value cannot be represented as a `long long`
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big or too small to fit into
+ *				a `long long`
  */
 - (long long)longLongValueWithBase: (unsigned char)base;
 
+/**
+ * @brief The value of the string in the specified base as an `unsigned char`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @param base The base to use. If the base is 0, base 16 is assumed if the
+ *	       string starts with 0x (after stripping white spaces). If the
+ *	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
+ *	       assumed.
+ * @return The value of the string in the specified base
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big to fit into an
+ *				`unsigned char`
+ */
+- (unsigned char)unsignedCharValueWithBase: (unsigned char)base;
+
+/**
+ * @brief The value of the string in the specified base as an `unsigned short`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @param base The base to use. If the base is 0, base 16 is assumed if the
+ *	       string starts with 0x (after stripping white spaces). If the
+ *	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
+ *	       assumed.
+ * @return The value of the string in the specified base
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big to fit into an
+ *				`unsigned short`
+ */
+- (unsigned short)unsignedShortValueWithBase: (unsigned char)base;
+
+/**
+ * @brief The value of the string in the specified base as an `unsigned int`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @param base The base to use. If the base is 0, base 16 is assumed if the
+ *	       string starts with 0x (after stripping white spaces). If the
+ *	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
+ *	       assumed.
+ * @return The value of the string in the specified base
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big to fit into an
+ *				`unsigned int`
+ */
+- (unsigned int)unsignedIntValueWithBase: (unsigned char)base;
+
+/**
+ * @brief The value of the string in the specified base as an `unsigned long`.
+ *
+ * Leading and trailing whitespaces are ignored.
+ *
+ * @param base The base to use. If the base is 0, base 16 is assumed if the
+ *	       string starts with 0x (after stripping white spaces). If the
+ *	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
+ *	       assumed.
+ * @return The value of the string in the specified base
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big to fit into an
+ *				`unsigned long`
+ */
+- (unsigned long)unsignedLongValueWithBase: (unsigned char)base;
+
 /**
  * @brief The value of the string in the specified base as an
  *	  `unsigned long long`.
  *
  * Leading and trailing whitespaces are ignored.
  *
- * If the string contains any non-number characters, an
- * @ref OFInvalidFormatException is thrown.
- *
- * If the number is too big to fit into an `unsigned long long`, an
- * @ref OFOutOfRangeException is thrown.
- *
  * @param base The base to use. If the base is 0, base 16 is assumed if the
- * 	       string starts with 0x (after stripping white spaces). If the
- * 	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
- * 	       assumed.
+ *	       string starts with 0x (after stripping white spaces). If the
+ *	       string starts with 0, base 8 is assumed. Otherwise, base 10 is
+ *	       assumed.
  * @return The value of the string in the specified base
- * @throw OFInvalidFormatException The string cannot be parsed as an
- *				   `unsigned long long`
- * @throw OFOutOfRangeException The value cannot be represented as an
+ * @throw OFInvalidFormatException The string contains non-number characters
+ * @throw OFOutOfRangeException The value is too big to fit into an
  *				`unsigned long long`
  */
 - (unsigned long long)unsignedLongLongValueWithBase: (unsigned char)base;
@@ -1385,7 +1635,7 @@ OF_ASSUME_NONNULL_END
 #import "OFString+XMLEscaping.h"
 #import "OFString+XMLUnescaping.h"
 
-#if !defined(NSINTEGER_DEFINED) && !__has_feature(modules)
+#if !defined(NSINTEGER_DEFINED) && !__has_feature(objc_modules)
 /*
  * Very *ugly* hack required for string boxing literals to work.
  *
